@@ -1,6 +1,6 @@
 package Text::CSV_XS;
 
-# Copyright (c) 2007-2007 H.Merijn Brand.  All rights reserved.
+# Copyright (c) 2007-2008 H.Merijn Brand.  All rights reserved.
 # Copyright (c) 1998-2001 Jochen Wiedmann. All rights reserved.
 # Portions Copyright (c) 1997 Alan Citterman. All rights reserved.
 #
@@ -28,12 +28,12 @@ use strict;
 use DynaLoader ();
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "0.27";
+$VERSION = "0.33";
 @ISA     = qw( DynaLoader );
 
-sub PV () { 0 }
-sub IV () { 1 }
-sub NV () { 2 }
+sub PV { 0 }
+sub IV { 1 }
+sub NV { 2 }
 
 # version
 #
@@ -50,116 +50,223 @@ sub version
 #   class/object method expecting no arguments and returning a reference to
 #   a newly created Text::CSV object.
 
-sub new ($;$)
+my %def_attr = (
+    quote_char		=> '"',
+    escape_char		=> '"',
+    sep_char		=> ',',
+    eol			=> '',
+    always_quote	=> 0,
+    binary		=> 0,
+    keep_meta_info	=> 0,
+    allow_loose_quotes	=> 0,
+    allow_loose_escapes	=> 0,
+    allow_whitespace	=> 0,
+    verbatim		=> 0,
+    types		=> undef,
+
+    _EOF		=> 0,
+    _STATUS		=> undef,
+    _FIELDS		=> undef,
+    _FFLAGS		=> undef,
+    _STRING		=> undef,
+    _ERROR_INPUT	=> undef,
+    );
+
+sub new
 {
     my $proto = shift;
     my $attr  = shift || {};
-    my $class = ref ($proto) || $proto;
-    my $self  = {
-	quote_char	=> '"',
-	escape_char	=> '"',
-	sep_char	=> ',',
-	eol		=> '',
-	always_quote	=> 0,
-	binary		=> 0,
-	keep_meta_info	=> 0,
-
-	_STATUS		=> undef,
-	_FIELDS		=> undef,
-	_FFLAGS		=> undef,
-	_STRING		=> undef,
-	_ERROR_INPUT	=> undef,
-
-	%$attr,
-	};
+    my $class = ref ($proto) || $proto	or return;
+    for (keys %{$attr}) {
+	m/^[a-z]/ && exists $def_attr{$_} and next;
+#	croak?
+#	print STDERR "### Cannot set attribute '$_'\n";
+	return;
+	}
+    my $self  = {%def_attr, %{$attr}};
     bless $self, $class;
-    exists $self->{types} and $self->types ($self->{types});
+    defined $self->{types} and $self->types ($self->{types});
     $self;
     } # new
 
+my %_cache_id = (	# Keep in sync with XS!
+    quote_char		=>  0,
+    escape_char		=>  1,
+    sep_char		=>  2,
+    binary		=>  3,
+    keep_meta_info	=>  4,
+    always_quote	=>  5,
+    allow_loose_quotes	=>  6,
+    allow_loose_escapes	=>  7,
+    allow_whitespace	=>  8,
+    allow_double_quoted	=>  9,
+
+    eol			=> 10,	# 10 .. 17
+    eol_len		=> 18,
+    eol_is_cr		=> 19,
+    has_types		=> 20,
+    verbatim		=> 21,
+    );
+sub _set_attr
+{
+    my ($self, $name, $val) = @_;
+    $self->{$name} = $val;
+    $self->{_CACHE} or return;
+    my @cache = unpack "C*", $self->{_CACHE};
+    $cache[$_cache_id{$name}] = defined $val ? ord $val : 0;
+    $self->{_CACHE} = pack "C*", @cache;
+    } # _set_attr
+
 # Accessor methods.
 #   It is unwise to change them halfway through a single file!
-sub quote_char ($;$)
+sub quote_char
 {
     my $self = shift;
-    @_ and $self->{quote_char} = shift;
+    @_ and $self->_set_attr ("quote_char", shift);
     $self->{quote_char};
     } # quote_char
 
-sub escape_char ($;$)
+sub escape_char
 {
     my $self = shift;
-    @_ and $self->{escape_char} = shift;
+    @_ and $self->_set_attr ("escape_char", shift);
     $self->{escape_char};
     } # escape_char
 
-sub sep_char ($;$)
+sub sep_char
 {
     my $self = shift;
-    @_ and $self->{sep_char} = shift;
+    @_ and $self->_set_attr ("sep_char", shift);
     $self->{sep_char};
     } # sep_char
 
-sub eol ($;$)
+sub eol
 {
     my $self = shift;
-    @_ and $self->{eol} = shift;
+    if (@_) {
+	my $eol = shift;
+	my $eol_len = length $eol;
+	$self->{eol} = $eol;
+	$self->{_CACHE} or return;
+	my @cache = unpack "C*", $self->{_CACHE};
+	if (($cache[$_cache_id{eol_len}] = $eol_len) < 8) {
+	    $cache[$_cache_id{eol_is_cr}] = $eol eq "\r" ? 1 : 0;
+	    }
+	else {
+	    $cache[$_cache_id{eol_is_cr}] = 0;
+	    }
+	$eol .= "\0\0\0\0\0\0\0\0";
+	$cache[$_cache_id{eol} + $_] = ord substr $eol, $_, 1 for 0 .. 7;
+	$self->{_CACHE} = pack "C*", @cache;
+	}
     $self->{eol};
     } # eol
 
-sub always_quote ($;$)
+sub always_quote
 {
     my $self = shift;
-    @_ and $self->{always_quote} = shift;
+    @_ and $self->_set_attr ("always_quote", shift);
     $self->{always_quote};
     } # always_quote
 
-sub binary ($;$)
+sub binary
 {
     my $self = shift;
-    @_ and $self->{binary} = shift;
+    @_ and $self->_set_attr ("binary", shift);
     $self->{binary};
     } # binary
 
-sub keep_meta_info ($;$)
+sub keep_meta_info
 {
     my $self = shift;
-    @_ and $self->{keep_meta_info} = shift;
+    @_ and $self->_set_attr ("keep_meta_info", shift);
     $self->{keep_meta_info};
     } # keep_meta_info
 
+sub allow_loose_quotes
+{
+    my $self = shift;
+    @_ and $self->_set_attr ("allow_loose_quotes", shift);
+    $self->{allow_loose_quotes};
+    } # allow_loose_quotes
+
+sub allow_loose_escapes
+{
+    my $self = shift;
+    @_ and $self->_set_attr ("allow_loose_escapes", shift);
+    $self->{allow_loose_escapes};
+    } # allow_loose_escapes
+
+sub allow_whitespace
+{
+    my $self = shift;
+    @_ and $self->_set_attr ("allow_whitespace", shift);
+    $self->{allow_whitespace};
+    } # allow_whitespace
+
+sub verbatim
+{
+    my $self = shift;
+    @_ and $self->_set_attr ("verbatim", shift);
+    $self->{verbatim};
+    } # verbatim
+
 # status
 #
 #   object method returning the success or failure of the most recent
 #   combine () or parse ().  there are no side-effects.
 
-sub status ($)
+sub status
 {
     my $self = shift;
     return $self->{_STATUS};
     } # status
 
+sub eof
+{
+    my $self = shift;
+    return $self->{_EOF};
+    } # status
+
 # error_input
 #
 #   object method returning the first invalid argument to the most recent
 #   combine () or parse ().  there are no side-effects.
 
-sub error_input ($)
+sub error_input
 {
     my $self = shift;
     return $self->{_ERROR_INPUT};
     } # error_input
 
+# erro_diag
+#
+#   If (and only if) an error occured, this function returns a code that
+#   indicates the reason of failure
+
+sub error_diag
+{
+    my $self = shift;
+    exists $self->{_ERROR_DIAG} or return;
+    my $diag = $self->{_ERROR_DIAG};
+    my $context = wantarray;
+    unless (defined $context) {	# Void context
+	print STDERR "# CSV_XS ERROR: ", 0 + $diag, " - $diag\n";
+	return;
+	}
+    return $context ? (0 + $diag, "$diag") : $diag;
+    } # error_diag
+
 # string
 #
 #   object method returning the result of the most recent combine () or the
 #   input to the most recent parse (), whichever is more recent.  there are
 #   no side-effects.
 
-sub string ($)
+sub string
 {
     my $self = shift;
-    return $self->{_STRING};
+    return ref $self->{_STRING} ? ${$self->{_STRING}} : undef;
     } # string
 
 # fields
@@ -168,7 +275,7 @@ sub string ($)
 #   input to the most recent combine (), whichever is more recent.  there
 #   are no side-effects.
 
-sub fields ($)
+sub fields
 {
     my $self = shift;
     return ref $self->{_FIELDS} ? @{$self->{_FIELDS}} : undef;
@@ -181,13 +288,13 @@ sub fields ($)
 #   are no side-effects. meta_info () returns (if available)  some of the
 #   field's properties
 
-sub meta_info ($)
+sub meta_info
 {
     my $self = shift;
     return ref $self->{_FFLAGS} ? @{$self->{_FFLAGS}} : undef;
     } # meta_info
 
-sub is_quoted ($$;$)
+sub is_quoted
 {
     my ($self, $idx, $val) = @_;
     ref $self->{_FFLAGS} &&
@@ -195,7 +302,7 @@ sub is_quoted ($$;$)
     $self->{_FFLAGS}[$idx] & 0x0001 ? 1 : 0;
     } # is_quoted
 
-sub is_binary ($$;$)
+sub is_binary
 {
     my ($self, $idx, $val) = @_;
     ref $self->{_FFLAGS} &&
@@ -214,17 +321,14 @@ sub is_binary ($$;$)
 #      setting string ()
 #      setting error_input ()
 
-sub combine ($@)
+sub combine
 {
-    my ($self, @part) = @_;
+    my $self = shift;
     my $str  = "";
-    my $ref  = \$str;
-    $self->{_FIELDS}      = \@part;
-    $self->{_FFLAGS}      = undef;
-    $self->{_ERROR_INPUT} = undef;
-    $self->{_STATUS}      =
-	(@part > 0) && $self->Combine (\$str, \@part, 0, $self->{eol});
-    $self->{_STRING}      = $str;
+    $self->{_FIELDS} = \@_;
+    $self->{_FFLAGS} = undef;
+    $self->{_STATUS} = (@_ > 0) && $self->Combine (\$str, \@_, 0);
+    $self->{_STRING} = \$str;
     $self->{_STATUS};
     } # combine
 
@@ -240,21 +344,24 @@ sub combine ($@)
 #      setting string ()
 #      setting error_input ()
 
-sub parse ($$)
+sub parse
 {
     my ($self, $str) = @_;
 
     my $fields = [];
     my $fflags = [];
-    $self->{_STRING} = $self->{ERROR_INPUT} = $str;
-    $self->{_STATUS} = 0;
-    $self->{_FIELDS} = undef;
-    $self->{_FFLAGS} = undef;
-    if (defined $str  && $self->Parse ($str, $fields, $fflags, 0)) {
+    $self->{_STRING} = \$str;
+    if (defined $str && $self->Parse ($str, $fields, $fflags)) {
+	$self->{_ERROR_INPUT} = undef;
 	$self->{_FIELDS} = $fields;
 	$self->{_FFLAGS} = $fflags;
 	$self->{_STATUS} = 1;
 	}
+    else {
+	$self->{_FIELDS} = undef;
+	$self->{_FFLAGS} = undef;
+	$self->{_STATUS} = 0;
+	}
     $self->{_STATUS};
     } # parse
 
@@ -265,7 +372,7 @@ sub types
     my $self = shift;
     if (@_) {
 	if (my $types = shift) {
-	    $self->{_types} = join "", map { chr $_ } @$types;
+	    $self->{_types} = join "", map { chr $_ } @{$types};
 	    $self->{types}  = $types;
 	    }
 	else {
@@ -302,12 +409,15 @@ Text::CSV_XS - comma-separated values manipulation routines
 
  $status       = $csv->status ();      # get the most recent status
  $bad_argument = $csv->error_input (); # get the most recent bad argument
+ $diag         = $csv->error_diag ();  # if an error occured, explains WHY
 
  $status = $csv->print ($io, $colref); # Write an array of fields
                                        # immediately to a file $io
  $colref = $csv->getline ($io);        # Read a line from file $io,
                                        # parse it and return an array
                                        # ref of fields
+ $eof = $csv->eof ();                  # Indicate if last parse or
+                                       # getline () hit End Of File
 
  $csv->types (\@t_array);              # Set column types
 
@@ -323,10 +433,10 @@ perhaps better called ASV (anything separated values) rather than just CSV.
 
 =head2 Embedded newlines
 
-B<Important Note>: The default behaviour is to only accept ascii characters.
+B<Important Note>: The default behavior is to only accept ascii characters.
 This means that fields can not contain newlines. If your data contains 
 newlines embedded in fields, or characters above 0x7e (tilde), or binary data,
-you *must* set C<binary => 1> in the call to C<new ()>.  To cover the widest
+you *must* set C<< binary => 1 >> in the call to C<new ()>.  To cover the widest
 range of parsing options, you will always want to set binary.
 
 But you still have the problem that you have to pass a correct line to the
@@ -334,12 +444,12 @@ C<parse ()> method, which is more complicated from the usual point of
 usage:
 
  my $csv = Text::CSV_XS->new ({ binary => 1, eol => $/ });
- while (<>) {
+ while (<>) {		#  WRONG!
      $csv->parse ($_);
      my @fields = $csv->fields ();
 
 will break, as the while might read broken lines, as that doesn't care
-about the quoting. If you need to support embedd newlines, the way to go
+about the quoting. If you need to support embedded newlines, the way to go
 is either
 
  use IO::Handle;
@@ -354,6 +464,88 @@ or, more safely in perl 5.6 and up
  while (my $row = $csv->getline ($io)) {
      my @fields = @$row;
  
+=head1 SPECIFICATION
+
+While no formal specification for CSV exists, RFC 4180 1) describes a common
+format and establishes "text/csv" as the MIME type registered with the IANA.
+
+Many informal documents exist that describe the CSV format. How To: The Comma
+Separated Value (CSV) File Format 2) provides an overview of the CSV format in
+the most widely used applications and explains how it can best be used and
+supported.
+
+ 1) http://tools.ietf.org/html/rfc4180
+ 2) http://www.creativyst.com/Doc/Articles/CSV/CSV01.htm
+
+The basic rules are as follows:
+
+B<CSV> is a delimited data format that has fields/columns separated by the comma
+character and records/rows separated by newlines. Fields that contain a special
+character (comma, newline, or double quote), must be enclosed in double quotes.
+However, if a line contains a single entry which is the empty string, it may be
+enclosed in double quotes. If a field's value contains a double quote character
+it is escaped by placing another double quote character next to it. The CSV file
+format does not require a specific character encoding, byte order, or line
+terminator format.
+
+=over 2
+
+=item *
+
+Each record is one line terminated by a line feed (ASCII/LF=0x0A) or a
+carriage return and line feed pair (ASCII/CRLF=0x0D 0x0A), however,
+line-breaks can be embedded.
+
+=item *
+
+Fields are separated by commas.
+
+=item *
+
+Allowable characters within a CSV field include 0x09 (tab) and the inclusive
+range of 0x20 (space) through 0x7E (tilde). In binary mode all characters
+are accepted, at least in quoted fields.
+
+=item *
+
+A field within CSV must be surrounded by double-quotes to contain a
+the separator character (comma).
+
+=back
+
+Though this is the most clear and restrictive definition, Text::CSV_XS is way
+more liberal than this, and allows extension:
+
+=over 2
+
+=item *
+
+Line termination by a single carriage return is accepted by default
+
+=item *
+
+The separation-, escape-, and escape- characters can be any ASCII character
+in the range from 0x20 (space) to 0x7E (tilde). Characters outside this range
+may or may not work as expected. Multibyte characters, like U+060c (ARABIC
+COMMA), U+FF0C (FULLWIDTH COMMA), U+241B (SYMBOL FOR ESCAPE), U+2424 (SYMBOL
+FOR NEWLINE), U+FF02 (FULLWIDTH QUOTATION MARK), and U+201C (LEFT DOUBLE
+QUOTATION MARK) (to give some examples of what might look promising) are
+therefor not allowed.
+
+=item *
+
+A field within CSV must be surrounded by double-quotes to contain an embedded
+double-quote, represented by a pair of consecutive double-quotes. In binary
+mode you may additionally use the sequence C<"0> for representation of a
+NULL byte.
+
+=item *
+
+Several violations of the above specification may be allowed by passing
+options to the object creator.
+
+=back
+
 =head1 FUNCTIONS
 
 =over 4
@@ -370,28 +562,73 @@ Currently the following attributes are available:
 
 =over 4
 
+=item eol
+
+An end-of-line string to add to rows, usually C<undef> (nothing,
+default), C<"\012"> (Line Feed) or C<"\015\012"> (Carriage Return,
+Line Feed). Cannot be longer than 7 (ASCII) characters.
+
+If both C<$/> and C<eol> equal C<"\015">, parsing lines that end on
+only a Carriage Return without Line Feed, will be C<parse>d correct.
+Line endings, whether in C<$/> or C<eol>, other than C<undef>,
+C<"\n">, C<"\r\n">, or C<"\r"> are not (yet) supported for parsing.
+
+=item sep_char
+
+The char used for separating fields, by default a comma. (C<,>).
+Limited to a single-byte character, usually in the range from 0x20
+(space) to 0x7e (tilde).
+
+The separation character can not be equal to the quote character.
+The separation character can not be equal to the escape character.
+
+=item allow_whitespace
+
+When this option is set to true, whitespace (TAB's and SPACE's)
+surrounding the separation character is removed when parsing. So
+lines like:
+
+  1 , "foo" , bar , 3 , zapp
+
+are now correctly parsed, even though it violates the CSV specs.
+Note that B<all> whitespace is stripped from start and end of each
+field. That would make is more a I<feature> than a way to be able
+to parse bad CSV lines, as
+
+ 1,   2.0,  3,   ape  , monkey
+
+will now be parsed as
+
+ ("1", "2.0", "3", "ape", "monkey")
+
+even if the original line was perfectly sane CSV.
+
 =item quote_char
 
 The char used for quoting fields containing blanks, by default the
 double quote character (C<">). A value of undef suppresses
 quote chars. (For simple cases only).
+Limited to a single-byte character, usually in the range from 0x20
+(space) to 0x7e (tilde).
 
 The quote character can not be equal to the separation character.
 
-=item eol
+=item allow_loose_quotes
 
-An end-of-line string to add to rows, usually C<undef> (nothing,
-default), C<"\012"> (Line Feed) or C<"\015\012"> (Carriage Return,
-Line Feed)
+By default, parsing fields that have C<quote_char> characters inside
+an unquoted field, like
 
-If both C<$/> and C<eol> equal C<"\015">, parsing lines that end on
-only a Carriage Return without Line Feed, will be C<parse>d correct.
-Line endings, wheather in C<$/> or C<eol>, other than C<undef>,
-C<"\n">, C<"\r\n">, or C<"\r"> are not (yet) supported for parsing.
+ 1,foo "bar" baz,42
+
+would result in a parse error. Though it is still bad practice to
+allow this format, we cannot help there are some vendors that make
+their applications spit out lines styled like this.
 
 =item escape_char
 
 The character used for escaping certain characters inside quoted fields.
+Limited to a single-byte character, usually in the range from 0x20
+(space) to 0x7e (tilde).
 
 The C<escape_char> defaults to being the literal double-quote mark (C<">)
 in other words, the same as the default C<quote_char>. This means that
@@ -406,12 +643,17 @@ the escape_char to be the same as what you changed the quote_char to.
 
 The escape character can not be equal to the separation character.
 
-=item sep_char
+=item allow_loose_escapes
 
-The char used for separating fields, by default a comma. (C<,>)
+By default, parsing fields that have C<escape_char> characters that
+escape characters that do not need to be escaped, like:
 
-The separation character can not be equal to the quote character.
-The separation character can not be equal to the escape character.
+ my $csv = Text::CSV_XS->new ({ escape_char => "\\" });
+ $csv->parse (qq{1,"my bar\'s",baz,42});
+
+would result in a parse error. Though it is still bad practice to
+allow this format, this option enables you to treat all escape character
+sequences equal.
 
 =item binary
 
@@ -443,6 +685,39 @@ true to be able to retrieve that information after parsing with
 the methods C<meta_info ()>, C<is_quoted ()>, and C<is_binary ()>
 described below.  Default is false.
 
+=item verbatim
+
+This is a quite controversial attribute to set, but it makes hard
+things possible.
+
+The basic thought behind this is to tell the parser that the normally
+special characters newline (NL) and Carriage Return (CR) will not be
+special when this flag is set, and be dealt with as being ordinary
+binary characters. This will ease working with data with embedded
+newlines.
+
+When C<verbatim> is used with C<getline ()>, getline
+auto-chomp's every line.
+
+Imagine a file format like
+
+  M^^Hans^Janssen^Klas 2\n2A^Ja^11-06-2007#\r\n
+
+where, the line ending is a very specific "#\r\n", and the sep_char
+is a ^ (caret). None of the fields is quoted, but embedded binary
+data is likely to be present. With the specific line ending, that
+shouldn't be too hard to detect.
+
+By default, Text::CSV_XS' parse function however is instructed to only
+know about "\n" and "\r" to be legal line endings, and so has to deal
+with the embedded newline as a real end-of-line, so it can scan the next
+line if binary is true, and the newline is inside a quoted field.
+With this attribute however, we can tell parse () to parse the line
+as if \n is just nothing more than a binary character.
+
+For parse () this means that the parser has no idea about line ending
+anymore, and getline () chomps line endings on reading.
+
 =back
 
 To sum it up,
@@ -452,13 +727,17 @@ To sum it up,
 is equivalent to
 
  $csv = Text::CSV_XS->new ({
-     quote_char     => '"',
-     escape_char    => '"',
-     sep_char       => ',',
-     eol            => '',
-     always_quote   => 0,
-     binary         => 0,
-     keep_meta_info => 0,
+     quote_char          => '"',
+     escape_char         => '"',
+     sep_char            => ',',
+     eol                 => '',
+     always_quote        => 0,
+     binary              => 0,
+     keep_meta_info      => 0,
+     allow_loose_quotes  => 0,
+     allow_loose_escapes => 0,
+     allow_whitespace    => 0,
+     verbatim            => 0,
      });
 
 For all of the above mentioned flags, there is an accessor method
@@ -538,6 +817,15 @@ by the function or undef for failure.
 The I<$csv-E<gt>string ()>, I<$csv-E<gt>fields ()> and I<$csv-E<gt>status ()>
 methods are meaningless, again.
 
+=item eof
+
+ $eof = $csv->eof ();
+
+If C<parse ()> or C<getline ()> was used with an IO stream, this
+method will return true (1) if the last call hit end of file, otherwise
+it will return false (''). This is useful to see the difference between
+a failure and end of file.
+
 =item types
 
  $csv->types (\@tref);
@@ -647,6 +935,26 @@ C<parse ()>, whichever was called more recently.
 This object function returns the erroneous argument (if it exists) of
 C<combine ()> or C<parse ()>, whichever was called more recently.
 
+=item error_diag
+
+ $csv->error_diag ();
+ $error_code  = 0  + $csv->error_diag ();
+ $error_str   = "" . $csv->error_diag ();
+ ($cde, $str) =      $csv->error_diag ();
+
+If (and only if) an error occured, this function returns the diagnostics
+of that error.
+
+If called in void context, it will print the internal error code and the
+associated error message to STDERR.
+
+If called in list context, it will return the error code and the error
+message in that order.
+
+If called in scalar context, it will return the diagnostics in a single
+scalar, a-la $!. It will contain the error code in numeric context, and
+the diagnostics message in string context.
+
 =back
 
 =head1 INTERNALS
@@ -704,49 +1012,30 @@ An example for parsing CSV lines:
       }
   else {
       my $err = $csv->error_input;
-      print "parse () failed on argument: ", $err, "\n";
+      print STDERR "parse () failed on argument: ", $err, "\n";
+      $csv->error_diag ();
       }
 
-=head1 CAVEATS
-
-This module is based upon a working definition of CSV format which may not be
-the most general.
-
-=over 4
-
-=item 1
-
-Allowable characters within a CSV field include 0x09 (tab) and the inclusive
-range of 0x20 (space) through 0x7E (tilde). In binary mode all characters
-are accepted, at least in quoted fields:
-
-=item 2
-
-A field within CSV may be surrounded by double-quotes. (The quote char)
-
-=item 3
-
-A field within CSV must be surrounded by double-quotes to contain a comma.
-(The separator char)
-
-=item 4
-
-A field within CSV must be surrounded by double-quotes to contain an embedded
-double-quote, represented by a pair of consecutive double-quotes. In binary
-mode you may additionally use the sequence C<"0> for representation of a
-NULL byte.
-
-=item 5
-
-A CSV string may be terminated by 0x0A (line feed) or by 0x0D,0x0A
-(carriage return, line feed).
-
-=back
-
 =head1 TODO
 
 =over 2
 
+=item More Errors & Warnings
+
+At current, it is hard to tell where or why an error occured (if
+at all). New extensions ought to be clear and concise in reporting
+what error occurred where and why, and possibly also tell a remedy
+to the problem. error_diag is a (very) good start, but there is more
+work to be done here.
+
+Basic calls should croak or warn on illegal parameters. Errors
+should be documented.
+
+If ->new () fails, there should be a way to obtain the reason why.
+Currently it is almost impossible to make new () fail, but in the
+future there should be a (default) option to have it fail when
+unsupported options are passed.
+
 =item eol
 
 Discuss an option to make the eol honor the $/ setting. Maybe
@@ -757,7 +1046,7 @@ is already enough, and new options only make things less opaque.
 
 =item setting meta info
 
-Future extensions might include extending the C<fields_flags ()>,
+Future extensions might include extending the C<meta_info ()>,
 C<is_quoted ()>, and C<is_binary ()> to accept setting these flags
 for fields, so you can specify which fields are quoted in the
 combine ()/string () combination.
@@ -770,7 +1059,7 @@ combine ()/string () combination.
 Adding an option that enables the parser to distinguish between
 empty fields and undefined fields, like
 
-  $csv->quote_always (1);
+  $csv->always_quote (1);
   $csv->allow_undef (1);
   $csv->parse (qq{,"",1,"2",,""});
   my @fld = $csv->fields ();
@@ -780,10 +1069,10 @@ of the current ("", "", "1", "2", "", "").
 
 =item combined methods
 
-Adding means (methods) that combine C<combine ()> and C<string ()> in
-a single call. Likewise for C<parse ()> and C<fields ()>. Given the
-trouble with embedded newlines, maybe just allowing C<getline ()> and
-C<print ()> is sufficient.
+Requests for adding means (methods) that combine C<combine ()> and
+C<string ()> in a single call will B<not> be honored. Likewise for
+C<parse ()> and C<fields ()>. Given the trouble with embedded newlines,
+using C<getline ()> and C<print ()> instead is the prefered way to go.
 
 =item Unicode
 
@@ -791,11 +1080,9 @@ Make C<parse ()> and C<combine ()> do the right thing for Unicode
 (UTF-8) if requested. See t/50_utf8.t. More complicated, but evenly
 important, also for C<getline ()> and C<print ()>.
 
-=item Space delimited seperators
-
-Discuss if and how C<Text::CSV_XS> should/could support formats like
-
-   1 , "foo" , "bar" , 3.19 ,
+Probably the best way to do this is to make a subclass
+Text::CSV_XS::Encoded that can be passed the required encoding and
+then behaves transparently (but slower).
 
 =item Double double quotes
 
@@ -803,7 +1090,8 @@ There seem to be applications around that write their dates like
 
    1,4,""12/11/2004"",4,1
 
-If we would support that, in what way?
+If we would support that, probably through allow_double_quoted
+Definitely belongs in t/65_allow.t
 
 =item Parse the whole file at once
 
@@ -825,11 +1113,98 @@ Returning something like
        },
      ]
 
+=item EBCDIC
+
+The hard-coding of characters and character ranges makes this module
+unusable on EBCDIC system. Using some #ifdef structure could enable
+these again without loosing speed. Testing would be the hard part.
+
+=back
+
+=head1 Release plan
+
+No guarantees, but this is what I have in mind right now:
+
+=over 2
+
+=item 0.33
+
+ - croak / carp
+ - error cause for failing new ()
+ - return undef
+ - DIAGNOSTICS setction in pod to *describe* the errors (see below)
+
+=item 0.34
+
+ - allow_double_quoted
+ - Text::CSV_XS::Encoded (maybe)
+
+=item 0.35
+
+ - csv2csv - a script to regenerate a CSV file to follow standards
+ - EBCDIC support
+
+=back
+
+=head1 DIAGNOSTICS
+
+Still under construction ...
+
+If an error occured, $csv->error_diag () can be used to get more information
+on the cause of the failure. Note that for speed reasons, the internal value
+is never cleared on success, so using the value returned by error_diag () in
+normal cases - when no error occured - may cause unexpected results.
+
+Currently these errors are available:
+
+=over 2
+
+=item 1001 "sep_char is equal to quote_char or escape_char"
+
+=item 2010 "ECR - QUO char inside quotes followed by CR not part of EOL"
+
+=item 2011 "ECR - Characters after end of quoted field"
+
+=item 2012 "EOF - End of data in parsing input stream"
+
+=item 2021 "EIQ - NL char inside quotes, binary off"
+
+=item 2022 "EIQ - CR char inside quotes, binary off"
+
+=item 2023 "EIQ - QUO ..."
+
+=item 2024 "EIQ - EOF cannot be escaped, not even inside quotes"
+
+=item 2025 "EIQ - Loose unescaped escape"
+
+=item 2026 "EIQ - Binary character inside quoted field, binary off"
+
+=item 2027 "EIQ - Quoted field not terminated"
+
+=item 2030 "EIF - NL char inside unquoted verbatim, binary off"
+
+=item 2031 "EIF - CR char is first char of field, not part of EOL"
+
+=item 2032 "EIF - CR char inside unquoted, not part of EOL"
+
+=item 2033 "EIF - QUO, QUO != ESC, binary off"
+
+=item 2034 "EIF - Loose unescaped quote"
+
+=item 2035 "EIF - Escaped EOF in unquoted field"
+
+=item 2036 "EIF - ESC error"
+
+=item 2037 "EIF - Binary character in unquoted field, binary off"
+
+=item 2110 "ECB - Binary character in Combine, binary off"
+
 =back
 
 =head1 SEE ALSO
 
-L<perl(1)>, L<IO::File(3)>, L<IO::Wrap(3)>, L<Spreadsheet::Read(3)>
+L<perl(1)>, L<IO::File(3)>, L{IO::Handle(3)>, L<IO::Wrap(3)>,
+L<Text::CSV(3)>, L<Text::CSV_PP(3)>.  and L<Spreadsheet::Read(3)>.
 
 =head1 AUTHORS and MAINTAINERS
 
@@ -851,7 +1226,7 @@ completed the documentation, fixed some RT bugs. See ChangeLog releases
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007-2007 H.Merijn Brand for PROCURA B.V.
+Copyright (C) 2007-2008 H.Merijn Brand for PROCURA B.V.
 Copyright (C) 1998-2001 Jochen Wiedmann. All rights reserved.
 Portions Copyright (C) 1997 Alan Citterman. All rights reserved.
 
